***********************************************************************
*     MODEL TO STUDY SATURATION OF SHEAR INSTABILITY         1999/03/24
*            LEGENDRE POLYNOMIALS FLOWS AND NEW TYPE DISTURBANCE
*            BY RUNGE-KUTTA METHOD
***********************************************************************
      PROGRAM MODEL

      IMPLICIT REAL*8(A-H,O-Z)

      CALL SAMAIN(3,-0.07D0, 11,'output-st.dat')

      END
************************************************************************
*     TEST MODEL FOR NEW ISPACK
************************************************************************
      SUBROUTINE SAMAIN(IPAT,APSI,IU,CF)

*     IPAT: ήP_n⡼ɤȤͿn
*     APSI: P_n⡼ɤο
*     IU: ϥեֹ
*     CF: ϥե̾
 
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER CF*(*)
      PARAMETER(PI=3.1415926535897932385D0)
      PARAMETER(MM=21)
      PARAMETER(LM=(MM+1)*(MM+1))
      PARAMETER(NB=23476)
      DIMENSION AVT(LM),W(LM*3)
      EXTERNAL SBDAVT,SBDISS

*---- OPEN SUBROUTINE PACKAGE AND INITIALIZE VARIABLES -----------------

      LEV=10                    !ⳬǴΥץ饷γ
      ITM=30                    !ȯŸ륹ƥå׿
      NDV=20                    !Runge-KuttaǤΥƥåʬ
      DT=1D0                    !եϤλֳִ

      DNU=10D0/(1D0*(MM*(MM+1)-2))**LEV

*/ 󷿤ξΥѥ᥿ /*

      DA=1D-3     !ο
      DB=100D0    !εտΥ롼Ȥѥ᥿
      X0=0        !濴֤Φ˺ɸ
      Y0=PI/4     !濴֤Φպɸ

      CALL SBOPEN(LEV,DNU,NDV,DT)
      CALL SBINIT(AVT,IPAT,APSI,DA,DB,X0,Y0)

*---- OPEN FHPACK ------------------------------------------------------

      CALL FHUOPN(IU,CF,'W',NB)

*---- TIME EVOLUTION BY RUNGE-KUTTA METHOD -----------------------------

      I=0
      TIM=0
      CALL FEPUTS(IU,LM,AVT)
      CALL SBCHCK(I,AVT)

      DO I=1,ITM
        CALL TDRKNU(LM,NDV,DT,TIM,AVT,W,SBDISS,SBDAVT)
        CALL FEPUTS(IU,LM,AVT)
        CALL SBCHCK(I,AVT)
      END DO

*---- CLOSE FHPACK -----------------------------------------------------

      CALL FHUCLS(IU)

      END
************************************************************************
*     OPEN SUBROUTINE PACKAGE
************************************************************************
      SUBROUTINE SBOPEN(LEV,DNU,NDV,DT)

      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(MM=21,JM=32,IM=64,LM=(MM+1)*(MM+1))
      PARAMETER(PI=3.1415926535897932385D0)
      PARAMETER(OMG=1)
      DIMENSION Q(JM*(MM+1)),R((MM+1)*(MM+1)),IT(5),T(IM*2)
      DIMENSION PSI(LM),AVT(LM),DAVT(LM)
      DIMENSION X(IM),Y(JM)
      DIMENSION P(JM*IM,4),DIS(MM+1)
      SAVE

      CALL STINIT(MM,JM,IM,Q,R,IT,T)
      CALL STOGRD(JM,IM,Y,X,Q)
      CALL NDDISI(MM,LEV,DNU*DT/(2*NDV),DIS)

      RETURN
*-----------------------------------------------------------------------
*     CALCULATION OF d(AVT)/dt
*-----------------------------------------------------------------------
      ENTRY SBDAVT(TIM,AVT,DAVT)

      CALL NDCA2P(MM,OMG,AVT,PSI)
      CALL STAJBA(MM,JM,IM,PSI,AVT,DAVT,P,Q,R,IT,T)

      DO L=1,LM
        DAVT(L)=-2*PI*DAVT(L)
      END DO

      RETURN
*-----------------------------------------------------------------------
*     CALCULATION OF THE EFFECT OF VISCOSITY
*-----------------------------------------------------------------------
      ENTRY SBDISS(TIM,DTIM,AVT)

      CALL NDDISA(MM,AVT,DIS)

      RETURN
*-----------------------------------------------------------------------
*     CHECK ENERGY AND ENSTROPHY CONSERVATION
*-----------------------------------------------------------------------
      ENTRY SBCHCK(I,AVT)

      CALL NDCA2P(MM,OMG,AVT,PSI)

      CALL NDGEEA(MM,PSI,ENE)
      CALL NDGENA(MM,PSI,ENS)
      WRITE(6,'(I5,2F15.10)') I,ENE,ENS

      RETURN
*-----------------------------------------------------------------------
*     INITIALIZATION OF AVT
*-----------------------------------------------------------------------
      ENTRY SBINIT(AVT,IPAT,APSI,DA,DB,X0,Y0)

*     / BASIC FIELD /

      N=IPAT
      CALL BSSET0(LM,PSI)
      PSI(N+1)=-N*(N+1)*APSI
      CALL NDTV2A(OMG,PSI)

*     / DISTURBANCE FIELD /

      CALL SCINID(JM,IM,P(1,2),X,Y,DA,DB,X0,Y0)
      CALL STG2SA(MM,JM,IM,P(1,2),AVT,P,Q,R,IT,T)

*     / BASIC FIELD + DISTURBANCE FIELD /

      DO L=1,MM+1
        AVT(L)=AVT(L)+PSI(L)
      END DO

      END
************************************************************************
*     CALCULATE DISTURBANCE FIELD
************************************************************************
      SUBROUTINE SCINID(JM,IM,G,X,Y,DA,DB,X0,Y0)

      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(PI=3.1415926535897932385D0)
      DIMENSION G(JM,IM)
      DIMENSION Y(JM),X(IM)

      CB=COS(PI/2-Y0)
      SB=SIN(PI/2-Y0)

      DO I=1,IM
        CA=COS(X(I)-X0)
        DO J=1,JM
          CC=COS(PI/2-Y(J))
          SC=SIN(PI/2-Y(J))
          CR=CB*CC+SB*SC*CA
          G(J,I)=DA*(EXP(DB*(CR-1))-(1-EXP(-2*DB))/(2*DB))
        END DO
      END DO

      END
